@@ -17,7 +17,7 @@ class CodeGenerator(object):
     """
 
     MAX_SIGNAL_STATES = 12
-    GENERATED_CODE_VERSION = "5.x"
+    GENERATED_CODE_VERSION = "6.x"
 
     def __init__(self, search_paths):
         self.search_paths = search_paths
@@ -36,7 +36,6 @@ class CodeGenerator(object):
         lines.extend(self._build_loop())
         lines.extend(self._build_commands())
         lines.extend(self._build_decoder())
-        lines.extend(self._build_filters())
 
         with open(os.path.join(os.path.dirname(__file__),
                 'signals.cpp.footer')) as footer:
@@ -101,8 +100,10 @@ class CodeGenerator(object):
             lines = []
             for message_index, message in enumerate(message_set.all_messages()):
                 if not message.active:
-                    LOG.warning("Skipping disabled message %s (0x%x)" %
-                            (message.name, message.id))
+                    msg = "Skipping disabled message 0x%x" % message.id
+                    if message.name is not None:
+                        msg += " (%s)" % message.name
+                    LOG.warning(msg)
                     continue
                 LOG.info("Added message '%s'" % message.name)
                 lines.append("        %s" % message)
@@ -144,40 +145,6 @@ class CodeGenerator(object):
 
         return lines
 
-    def _build_filters(self):
-        # These arrays can't be initialized when we create the variables or else
-        # they end up in the .data portion of the compiled program, and it
-        # becomes too big for the microcontroller. Initializing them at runtime
-        # gets around that problem.
-        lines = []
-        lines.append("CanFilter FILTERS[MAX_MESSAGE_COUNT];")
-
-        lines.append("")
-        lines.append("CanFilter* openxc::signals::initializeFilters("
-                "uint64_t address, int* count) {")
-
-        def block(message_set):
-            lines = []
-            lines.append("        switch(address) {")
-            for bus in message_set.valid_buses():
-                lines.append("        case %s:" % bus.controller)
-                if bus.raw_can_mode == "unfiltered":
-                    lines.append("            *count = 0;")
-                else:
-                    lines.append("            *count = %d;" % len(
-                            list(bus.active_messages())))
-                    for i, message in enumerate(bus.active_messages()):
-                        lines.append("            FILTERS[%d] = {%d, 0x%x, %d};" % (
-                                i, i, message.id, 1))
-                lines.append("            break;")
-            lines.append("        }")
-            return lines
-
-        lines.extend(self._message_set_switcher(block))
-        lines.append("    return FILTERS;")
-        lines.append("}")
-        return lines
-
     def _build_signal_states(self):
         lines = []
         lines.append("const int MAX_SIGNAL_STATES = %d;" %
@@ -261,12 +228,16 @@ class CodeGenerator(object):
 
     def _build_initializers(self):
         lines = []
-        lines.append("void openxc::signals::initialize() {")
+        lines.append("void openxc::signals::initialize(openxc::diagnostics::DiagnosticsManager* diagnosticsManager) {")
 
         def block(message_set):
-            return ["        %s();" % initializer
+            init_lines = ["        %s();" % initializer
                 for initializer in message_set.initializers]
+            init_lines.extend(["        %s" % message
+                for message in message_set.active_diagnostic_messages()])
+            return init_lines
         lines.extend(self._message_set_switcher(block))
+
         lines.append("}")
         lines.append("")
         return lines
@@ -302,38 +273,36 @@ class CodeGenerator(object):
     def _build_decoder(self):
         lines = []
         lines.append("void openxc::signals::decodeCanMessage("
-                "Pipeline* pipeline, CanBus* bus, int id, uint64_t data) {")
+                "Pipeline* pipeline, CanBus* bus, CanMessage* message) {")
 
         def block(message_set):
             lines = []
             lines.append(" " * 8 + "switch(bus->address) {")
             for bus in message_set.valid_buses():
                 lines.append(" " * 8 + "case %s:" % bus.controller)
-                lines.append(" " * 12 + "switch (id) {")
+                lines.append(" " * 12 + "switch (message->id) {")
                 for message in bus.active_messages():
                     if (len(list(message.active_signals())) > 0 or
                             len(list(message.handlers)) > 0):
                         lines.append(" " * 12 + "case 0x%x: // %s" % (message.id,
                                 message.name))
                         for handler in message.handlers:
-                            lines.append(" " * 16 + "%s(id, data, SIGNALS[%d], " % (
+                            lines.append(" " * 16 + "%s(message, SIGNALS[%d], " % (
                                 handler, message_set.index) +
                                     "getSignalCount(), pipeline);")
                         for signal in message.active_signals():
                             line = " " * 16
-                            line += ("can::read::translateSignal(pipeline, "
-                                        "&SIGNALS[%d][%d], data, " %
+                            line += ("can::read::translateSignal("
+                                        "&SIGNALS[%d][%d], message, " %
                                         (message_set.index, signal.array_index))
-                            if signal.handler:
-                                line += "&%s, " % signal.handler
-                            line += ("SIGNALS[%d], getSignalCount()); // %s" % (
+                            line += ("SIGNALS[%d], getSignalCount(), pipeline); // %s" % (
                                 message_set.index, signal.name))
                             lines.append(line)
                         lines.append("                break;")
                 lines.append("            }")
                 if bus.raw_can_mode != "off":
                     lines.append(" " * 12 + "openxc::can::read::passthroughMessage("
-                            "bus, id, data, getMessages(), getMessageCount(), pipeline);")
+                            "bus, message, getMessages(), getMessageCount(), pipeline);")
                 lines.append("            break;")
             lines.append("        }")
             return lines
