@@ -12,22 +12,22 @@ class Network(object):
         self.messages = {}
 
         for message_id, message in all_messages.items():
-            message_id = int(message_id, 0)
+            numeric_message_id = int(message_id, 0)
             query = "./Node/TxMessage[ID=\"0x%s\"]"
             # Search for both lower and upper case hex
             for attr_value in ["%X", "%x"]:
-                node = tree.find(query % (attr_value % message_id))
+                node = tree.find(query % (attr_value % numeric_message_id))
                 if node is not None:
                     break
             if node is None:
-                LOG.warning("Unable to find message ID 0x%x in %s" % (message_id,
-                  database_name))
+                LOG.warning("Unable to find message ID 0x%x in %s" % (
+                    numeric_message_id, database_name))
             else:
-                self.messages[message_id] = XMLBackedMessage.from_xml_node(
-                    node, message['signals'])
+                self.messages[numeric_message_id] = XMLBackedMessage.from_xml_node(
+                    node, message_id, message['signals'])
 
     def to_dict(self):
-        return {'messages': dict(("0x%x" % message.id,
+        return {'messages': dict(("%s" % message.string_id,
                     message.to_dict())
                 for message in list(self.messages.values())
                 if len(message.signals) > 0)}
@@ -35,18 +35,21 @@ class Network(object):
 class XMLBackedMessage(Message):
 
     @classmethod
-    def from_xml_node(cls, node, mapped_signals):
+    def from_xml_node(cls, node, string_message_id, mapped_signals):
         message = cls()
 
         message.name = node.find("Name").text
+        # Keep around whatever string is used in the mapping, so our dict merge
+        # works with case insensitive key matching
+        message.string_id = string_message_id
         message.id = int(node.find("ID").text, 0)
 
-        for signal_name in mapped_signals.keys():
+        for signal_name, mapped_signal in mapped_signals.iteritems():
             mapped_signal_node = node.find("Signal[Name=\"%s\"]" % signal_name)
             if mapped_signal_node is not None:
-                mapped_signal = XMLBackedSignal.from_xml_node(mapped_signal_node)
-                mapped_signal.generic_name = signal_name
-                message.signals[signal_name] = mapped_signal
+                signal = XMLBackedSignal.from_xml_node(mapped_signal_node)
+                signal.generic_name = mapped_signal.get('generic_name', None) or signal_name
+                message.signals[signal_name] = signal
 
         return message
 
