@@ -1,8 +1,13 @@
 """Abstract base interface for vehicle data sources."""
+from __future__ import print_function
+
 import threading
 import logging
+import binascii
 import google.protobuf.message
 import string
+import sys
+import datetime
 from google.protobuf.internal.decoder import _DecodeVarint
 
 from openxc import openxc_pb2
@@ -20,7 +25,7 @@ class DataSource(threading.Thread):
     A data source requires a callback method to be specified. Whenever new data
     is received, it will pass it to that callback.
     """
-    def __init__(self, callback=None):
+    def __init__(self, callback=None, log_mode=None):
         """Construct a new DataSource.
 
         By default, DataSource threads are marked as ``daemon`` threads, so they
@@ -33,8 +38,13 @@ class DataSource(threading.Thread):
         super(DataSource, self).__init__()
         self.callback = callback
         self.daemon = True
+        self.logger = SourceLogger(self, log_mode)
 
-    def _read(self, timeout=None):
+    def start(self):
+        self.logger.start()
+        super(DataSource, self).start()
+
+    def read(self, timeout=None):
         """Read data from the source.
 
         Kwargs:
@@ -43,20 +53,39 @@ class DataSource(threading.Thread):
         """
         raise NotImplementedError("Don't use DataSource directly")
 
+    def read_logs(self, timeout=None):
+        """Read log data from the source.
 
-class BytestreamDataSource(DataSource):
-    """A source that receives data is a series of bytes, with discrete messages
-    separated by a newline character.
+        Kwargs:
+            timeout (float) - if the source implementation could potentially
+                block, timeout after this number of seconds.
+        """
+        raise NotImplementedError("Don't use DataSource directly")
 
-    Subclasses of this class need only to implement the ``_read`` method.
-    """
 
-    MAX_PROTOBUF_MESSAGE_LENGTH = 200
+class SourceLogger(threading.Thread):
+    FILENAME_TEMPLATE = "%d-%m-%Y.%H-%M-%S"
 
-    def __init__(self, callback=None):
-        super(BytestreamDataSource, self).__init__(callback)
-        self.bytes_received = 0
-        self.corrupted_messages = 0
+    def __init__(self, source, mode="off"):
+        super(SourceLogger, self).__init__()
+        self.daemon = True
+        self.source = source
+        self.mode = mode
+        self.file = None
+
+        if self.mode == "file":
+            filename = "openxc-logs-%s.txt" % datetime.datetime.now().strftime(
+                    self.FILENAME_TEMPLATE)
+            self.file = open(filename, 'wa')
+
+    def record(self, message):
+        if self.mode != "off" and len(message) > 0:
+            log_file = None
+            if self.mode == "stderr":
+                log_file = sys.stderr
+            elif self.mode == "file" and self.file is not None:
+                log_file = self.file
+            print("LOG: %s" % message, file=log_file)
 
     def run(self):
         """Continuously read data from the source and attempt to parse a valid
@@ -66,7 +95,55 @@ class BytestreamDataSource(DataSource):
         message_buffer = b""
         while True:
             try:
-                message_buffer += self._read()
+                message_buffer += self.source.read_logs()
+            except DataSourceError as e:
+                LOG.warn("Can't read logs from data source -- stopping: %s", e)
+                break
+            except NotImplementedError as e:
+                LOG.info("%s doesn't support logging" % self)
+                break
+
+            while True:
+                if "\x00" not in message_buffer:
+                    break
+                record, _, remainder = message_buffer.partition(b"\x00")
+                self.record(record)
+                message_buffer = remainder
+
+
+class BytestreamDataSource(DataSource):
+    """A source that receives data is a series of bytes, with discrete messages
+    separated by a newline character.
+
+    Subclasses of this class need only to implement the ``read`` method.
+    """
+
+    MAX_PROTOBUF_MESSAGE_LENGTH = 200
+
+    def __init__(self, callback=None, log_mode=None):
+        super(BytestreamDataSource, self).__init__(callback, log_mode)
+        self.bytes_received = 0
+        self.corrupted_messages = 0
+
+    def _message_valid(self, message):
+        if not hasattr(message, '__iter__'):
+            return False
+        if not ('name' in message and 'value' in message or
+                    ('id' in message and 'data' in message) or
+                    ('id' in message and 'bus' in message) or
+                    'command_response' in message):
+            return False
+        return True
+
+    def run(self):
+        """Continuously read data from the source and attempt to parse a valid
+        message from the buffer of bytes. When a message is parsed, passes it
+        off to the callback if one is set.
+        """
+        message_buffer = b""
+        while True:
+            try:
+                message_buffer += self.read()
             except DataSourceError as e:
                 LOG.warn("Can't read from data source -- stopping: %s", e)
                 break
@@ -76,52 +153,79 @@ class BytestreamDataSource(DataSource):
                         message_buffer)
                 if message is None:
                     break
-                if not hasattr(message, '__iter__') or not (
-                        ('name' in message and 'value' in message) or (
-                        'id' in message and 'data' in message)):
+                if not self._message_valid(message):
                     self.corrupted_messages += 1
                     break
 
                 self.bytes_received += byte_count
                 if self.callback is not None:
                     self.callback(message)
+                self._receive_command_response(message)
 
     def _protobuf_to_dict(self, message):
         parsed_message = {}
         if message.type == message.RAW and message.HasField('raw_message'):
-            if message.raw_message.HasField('bus'):
-                parsed_message['bus'] = message.raw_message.bus
-            if message.raw_message.HasField('message_id'):
-                parsed_message['id'] = message.raw_message.message_id
-            if message.raw_message.HasField('data'):
-                parsed_message['data'] = "0x%x" % message.raw_message.data
+            raw_message = message.raw_message
+            if raw_message.HasField('bus'):
+                parsed_message['bus'] = raw_message.bus
+            if raw_message.HasField('message_id'):
+                parsed_message['id'] = raw_message.message_id
+            if raw_message.HasField('data'):
+                parsed_message['data'] = "0x%s" % binascii.hexlify(raw_message.data)
+        elif message.type == message.DIAGNOSTIC:
+            diagnostic_message = message.diagnostic_message
+            if diagnostic_message.HasField('bus'):
+                parsed_message['bus'] = diagnostic_message.bus
+            if diagnostic_message.HasField('message_id'):
+                parsed_message['id'] = diagnostic_message.message_id
+            if diagnostic_message.HasField('mode'):
+                parsed_message['mode'] = diagnostic_message.mode
+            if diagnostic_message.HasField('pid'):
+                parsed_message['pid'] = diagnostic_message.pid
+            if diagnostic_message.HasField('success'):
+                parsed_message['success'] = diagnostic_message.success
+            if diagnostic_message.HasField('negative_response_code'):
+                parsed_message['negative_response_code'] = diagnostic_message.negative_response_code
+            if diagnostic_message.HasField('payload'):
+                parsed_message['payload'] = "0x%s" % binascii.hexlify(diagnostic_message.payload)
         elif message.type == message.TRANSLATED:
-            parsed_message['name'] = message.translated_message.name
-            if message.translated_message.HasField('numeric_event'):
-                parsed_message['event'] = message.translated_message.numeric_event
-            elif message.translated_message.HasField('boolean_event'):
-                parsed_message['event'] = message.translated_message.boolean_event
-            elif message.translated_message.HasField('string_event'):
-                parsed_message['event'] = message.translated_message.string_event
+            translated_message = message.translated_message
+            parsed_message['name'] = translated_message.name
+            if translated_message.HasField('numeric_event'):
+                parsed_message['event'] = translated_message.numeric_event
+            elif translated_message.HasField('boolean_event'):
+                parsed_message['event'] = translated_message.boolean_event
+            elif translated_message.HasField('string_event'):
+                parsed_message['event'] = translated_message.string_event
 
-            if message.translated_message.HasField('numeric_value'):
-                parsed_message['value'] = message.translated_message.numeric_value
-            elif message.translated_message.HasField('boolean_value'):
-                parsed_message['value'] = message.translated_message.boolean_value
-            elif message.translated_message.HasField('string_value'):
-                parsed_message['value'] = message.translated_message.string_value
+            if translated_message.HasField('numeric_value'):
+                parsed_message['value'] = translated_message.numeric_value
+            elif translated_message.HasField('boolean_value'):
+                parsed_message['value'] = translated_message.boolean_value
+            elif translated_message.HasField('string_value'):
+                parsed_message['value'] = translated_message.string_value
             else:
                 parsed_message = None
         else:
             parsed_message = None
         return parsed_message
 
+    def _receive_command_response(self, message):
+        # TODO the controller/source are getting a litlte mixed up since the
+        # controller now needs to receive responses from the soruce side, maybe
+        # just mix them again. the only exception to being both is a trace
+        # source, and we can just leave the controller methods on that
+        # unimplemented
+        self.open_requests = getattr(self, 'open_requests', [])
+        for open_request in self.open_requests:
+            open_request.put(message)
+
     def _parse_json_message(self, message_buffer):
         parsed_message = None
         remainder = message_buffer
         message = ""
-        if b"\n" in message_buffer:
-            message, _, remainder = message_buffer.partition(b"\n")
+        if b"\x00" in message_buffer:
+            message, _, remainder = message_buffer.partition(b"\x00")
             try:
                 parsed_message = JsonFormatter.deserialize(message)
                 if not isinstance(parsed_message, dict):
@@ -182,7 +286,9 @@ class BytestreamDataSource(DataSource):
         if not isinstance(message_buffer, bytes):
             message_buffer = message_buffer.encode("utf-8")
 
-        if all((char in string.printable for char in message_buffer)):
+        json_chars = ['\x00']
+        json_chars.extend(string.printable)
+        if all((char in json_chars for char in message_buffer)):
             return self._parse_json_message(message_buffer)
         else:
             return self._parse_protobuf_message(message_buffer)
